from django.conf import settings


BASE_APP_ICONS = {
    'auth': 'bi bi-person-badge',
    'api': 'bi bi-bezier', # for any app named api
    'blog': 'bi bi-journal-richtext', # for any app named blog
    'shop': 'bi bi-shop', # for any app named shop
    'store': 'bi bi-shop', # for any app named store
    'core': 'bi bi-cpu', # for any app named core
    'contacts': 'bi bi-person-lines-fill', # for any app named contacts
    'accounts': 'bi bi-person-rolodex', # for any app named accounts
    'inventory': 'bi bi-clipboard2', # for any app named inventory
    'chat': 'bi bi-chat-text', # for any app named inventory
    'messages': 'bi bi-chat-right-text', # for any app named inventory
}


BASE_MODEL_ICONS = {
    'groups': 'bi bi-people-fill',
    'users': 'bi bi-person-fill',
}


global PALETTE_SETTINGS

try:
    PALETTE_SETTINGS:dict = settings.PALETTE_ADMIN_SETTINGS
except Exception as e:
    PALETTE_SETTINGS = dict()

SETTINGS = {
  'site_header': 'Dj Palette',
  'site_title': 'Dj Palette',
  'site_icon': 'bi bi-palette',
  'index_title': 'Dashboard',
  'custom_urls': [],
  'custom_links': [
        {
            'label': "Support",
            'url': 'admin:support_chat', # a name from a valid url pattern,
            'left_icon': 'bi bi-support-chat', # or right_icon for positioning of the icon
        }
    ]
}

SETTINGS.update({
    'app_icons': {**BASE_APP_ICONS, **PALETTE_SETTINGS.get('app_icons', dict())},
    'model_icons': {**BASE_MODEL_ICONS, **PALETTE_SETTINGS.get('model_icons', dict())},
})



def get_base_app_icon(app_label):
    _settings = SETTINGS
    return SETTINGS['app_icons'][app_label]

def get_base_model_icon(model_name, app_label=None):
    _settings = SETTINGS
    return SETTINGS['model_icons'].get(model_name, '')





# Generated by Django 5.2.8 on 2025-11-23 08:06

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Component',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(blank=True, max_length=200, null=True)),
                ('components', models.ManyToManyField(blank=True, to='palette_admin.component')),
            ],
        ),
        migrations.CreateModel(
            name='AdminPage',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(blank=True, max_length=200, null=True)),
                ('route', models.CharField(blank=True, max_length=200, null=True)),
                ('content', models.TextField(blank=True, null=True)),
                ('components', models.ManyToManyField(blank=True, to='palette_admin.component')),
            ],
        ),
    ]



from django import template

register = template.Library()


@register.filter
def app_icon_class(app, icon=''):
    from dj_palette.palette_admin.conf import get_base_app_icon
    label = app['app_label']
    if label:
        return get_base_app_icon(label)
    return icon


@register.filter
def model_icon_class(model, icon=''):
    from dj_palette.palette_admin.conf import get_base_model_icon

    label:str = model['name'].lower()
    if label:
        return get_base_model_icon(label)
    return icon


@register.filter
def action_label(choice_label) -> str:
    label = choice_label.split(" ")[0].strip()
    return label


@register.filter('getattr')
def get_obj_attr(obj, attr):
    # print(obj.keys())
    return getattr(obj, attr, None)




from django.db import models

# Create your models here.
class AdminPage(models.Model):
	name=models.CharField(max_length=200, blank=True, null=True)
	route=models.CharField(max_length=200, blank=True, null=True)
	components=models.ManyToManyField("Component", blank=True)
	content=models.TextField(blank=True, null=True)

class Component(models.Model):
	name=models.CharField(max_length=200, blank=True, null=True)
	components=models.ManyToManyField("self", blank=True)




from django.contrib import admin
from .site import PaletteAdminSite
from django.contrib.auth.models import Group
from django.contrib.auth import get_user_model
from django.db import models
from django.contrib.admin.utils import model_format_dict
from django.utils.translation import gettext as _
from django.utils.translation import ngettext
from django.http import HttpResponseRedirect
from django.template.response import SimpleTemplateResponse, TemplateResponse
from django.contrib.admin.exceptions import DisallowedModelAdminToField
from django.contrib import messages
from django.core.exceptions import (
    PermissionDenied
)
from django.contrib.admin.utils import (
    flatten_fieldsets,
    model_format_dict,
    model_ngettext,
    quote,
    unquote,
)
from django.db import models, router, transaction
from django.forms.formsets import all_valid
from django.utils.decorators import method_decorator
from django.views.decorators.csrf import csrf_protect
from functools import partial, update_wrapper
from django.contrib.admin import helpers, widgets





csrf_protect_m = method_decorator(csrf_protect)

class IncorrectLookupParameters(Exception):
    pass


User = get_user_model()

IS_POPUP_VAR = "_popup"
TO_FIELD_VAR = "_to_field"
IS_FACETS_VAR = "_facets"


class PaletteModelAdmin(admin.ModelAdmin):
    grid_display = ("__str__",)
    grid_display_links = ("__str__", )
    detail_view_template = None

    def get_urls(self):
        from django.urls import path
        urls:list = super().get_urls()

        
        def wrap(view):
            def wrapper(*args, **kwargs):
                return self.admin_site.admin_view(view)(*args, **kwargs)

            wrapper.model_admin = self
            return update_wrapper(wrapper, view)

        info = self.opts.app_label, self.opts.model_name


        urls += [
            path(
                "<path:object_id>/",
                wrap(self.detail_view),
                name="%s_%s_view" % info,
            ),
        ]
        return urls

    
    def get_grid_display(self, request):
        """
        Return a sequence containing the fields to be displayed on the
        changelist.
        """
        return self.grid_display


    def get_grid_display_links(self, request, grid_display):
        """
        Return a sequence containing the fields to be displayed as links
        on the changelist. The list_display parameter is the list of fields
        returned by get_grid_display().
        """
        if (
            self.grid_display_links
            or self.grid_display_links is None
            or not grid_display
        ):
            return self.grid_display_links
        else:
            # Use only the first item in list_display as link
            return list(grid_display)[:1]
    
    
    def get_sortable_by(self, request):
        """Hook for specifying which fields can be sorted in the changelist."""
        return (
            self.sortable_by
            if self.sortable_by is not None
            else (self.get_list_display(request), self.get_grid_display(request))
        )

    def detail_view(self, request, **kwargs):
        return 

    
    def get_changelist_instance(self, request):
        """
        Return a `ChangeList` instance based on `request`. May raise
        `IncorrectLookupParameters`.
        """
        list_display = self.get_list_display(request)
        list_display_links = self.get_list_display_links(request, list_display)
        grid_display = self.get_grid_display(request)
        grid_display_links = self.get_grid_display_links(request, list_display)
        # Add the action checkboxes if any actions are available.
        if self.get_actions(request):
            list_display = ["action_checkbox", *list_display]
        sortable_by = self.get_sortable_by(request)
        ChangeList = self.get_changelist(request)
        return ChangeList(
            request,
            self.model,
            list_display,
            list_display_links,
            grid_display,
            grid_display_links,
            self.get_list_filter(request),
            self.date_hierarchy,
            self.get_search_fields(request),
            self.get_list_select_related(request),
            self.list_per_page,
            self.list_max_show_all,
            self.list_editable,
            self,
            sortable_by,
            self.search_help_text,
        )

    
    
    @csrf_protect_m
    def changelist_view(self, request, extra_context=None):
        """
        The 'change list' admin view for this model.
        """
        from django.contrib.admin.views.main import ERROR_FLAG

        app_label = self.opts.app_label
        if not self.has_view_or_change_permission(request):
            raise PermissionDenied

        try:
            cl = self.get_changelist_instance(request)
        except IncorrectLookupParameters:
            # Wacky lookup parameters were given, so redirect to the main
            # changelist page, without parameters, and pass an 'invalid=1'
            # parameter via the query string. If wacky parameters were given
            # and the 'invalid=1' parameter was already in the query string,
            # something is screwed up with the database, so display an error
            # page.
            if ERROR_FLAG in request.GET:
                return SimpleTemplateResponse(
                    "admin/invalid_setup.html",
                    {
                        "title": _("Database error"),
                    },
                )
            return HttpResponseRedirect(request.path + "?" + ERROR_FLAG + "=1")

        # If the request was POSTed, this might be a bulk action or a bulk
        # edit. Try to look up an action or confirmation first, but if this
        # isn't an action the POST will fall through to the bulk edit check,
        # below.
        action_failed = False
        selected = request.POST.getlist(helpers.ACTION_CHECKBOX_NAME)

        actions = self.get_actions(request)
        # Actions with no confirmation
        if (
            actions 
            and request.method == "POST"
            and "index" in request.POST
            and "_save" not in request.POST
        ):
            if selected:
                response = self.response_action(
                    request, queryset=cl.get_queryset(request)
                )
                if response:
                    return response
                else:
                    action_failed = True
            else:
                msg = _(
                    "Items must be selected in order to perform "
                    "actions on them. No items have been changed."
                )
                self.message_user(request, msg, messages.WARNING)
                action_failed = True

        # Actions with confirmation
        if (
            actions
            and request.method == "POST"
            and helpers.ACTION_CHECKBOX_NAME in request.POST
            and "index" not in request.POST
            and "_save" not in request.POST
        ):
            if selected:
                response = self.response_action(
                    request, queryset=cl.get_queryset(request)
                )
                if response:
                    return response
                else:
                    action_failed = True

        if action_failed:
            # Redirect back to the changelist page to avoid resubmitting the
            # form if the user refreshes the browser or uses the "No, take
            # me back" button on the action confirmation page.
            return HttpResponseRedirect(request.get_full_path())

        # If we're allowing changelist editing, we need to construct a formset
        # for the changelist given all the fields to be edited. Then we'll
        # use the formset to validate/process POSTed data.
        formset = cl.formset = None

        # Handle POSTed bulk-edit data.
        if request.method == "POST" and cl.list_editable and "_save" in request.POST:
            print("Add Form Submitted")

            if not self.has_change_permission(request):
                raise PermissionDenied
            FormSet = self.get_changelist_formset(request)
            modified_objects = self._get_list_editable_queryset(
                request, FormSet.get_default_prefix()
            )
            formset = cl.formset = FormSet(
                request.POST, request.FILES, queryset=modified_objects
            )
            if formset.is_valid():
                changecount = 0
                with transaction.atomic(using=router.db_for_write(self.model)):
                    for form in formset.forms:
                        if form.has_changed():
                            obj = self.save_form(request, form, change=True)
                            self.save_model(request, obj, form, change=True)
                            self.save_related(request, form, formsets=[], change=True)
                            change_msg = self.construct_change_message(
                                request, form, None
                            )
                            self.log_change(request, obj, change_msg)
                            changecount += 1
                if changecount:
                    msg = ngettext(
                        "%(count)s %(name)s was changed successfully.",
                        "%(count)s %(name)s were changed successfully.",
                        changecount,
                    ) % {
                        "count": changecount,
                        "name": model_ngettext(self.opts, changecount),
                    }
                    self.message_user(request, msg, messages.SUCCESS)

                return HttpResponseRedirect(request.get_full_path())

        # Handle GET -- construct a formset for display.
        elif cl.list_editable and self.has_change_permission(request):
            FormSet = self.get_changelist_formset(request)
            formset = cl.formset = FormSet(queryset=cl.result_list)

        # Build the list of media to be used by the formset.
        if formset:
            media = self.media + formset.media
        else:
            media = self.media

        # Build the action form and populate it with available actions.
        if actions:
            action_form = self.action_form(auto_id=None)
            action_form.fields["action"].choices = self.get_action_choices(request)
            media += action_form.media
        else:
            action_form = None

        selection_note_all = ngettext(
            "%(total_count)s selected", "All %(total_count)s selected", cl.result_count
        )

        context = {
            **self.admin_site.each_context(request),
            "module_name": str(self.opts.verbose_name_plural),
            "selection_note": _("0 of %(cnt)s selected") % {"cnt": len(cl.result_list)},
            "selection_note_all": selection_note_all % {"total_count": cl.result_count},
            "title": cl.title,
            "subtitle": None,
            "is_popup": cl.is_popup,
            "to_field": cl.to_field,
            "cl": cl,
            "media": media,
            "has_add_permission": self.has_add_permission(request),
            "opts": cl.opts,
            "action_form": action_form,
            "actions_on_top": self.actions_on_top,
            "actions_on_bottom": self.actions_on_bottom,
            "actions_selection_counter": self.actions_selection_counter,
            "preserved_filters": self.get_preserved_filters(request),
            **(extra_context or {}),
        }

        request.current_app = self.admin_site.name

        return TemplateResponse(
            request,
            self.change_list_template
            or [
                "admin/%s/%s/change_list.html" % (app_label, self.opts.model_name),
                "admin/%s/change_list.html" % app_label,
                "admin/change_list.html",
            ],
            context,
        )

    
    def _changeform_view(self, request, object_id, form_url, extra_context):
        if request.method == "POST":
            # c5e536d0-8a26-4b08-b3d7-86414b9b7a9c
            print("Form Submission!")

        to_field = request.POST.get(TO_FIELD_VAR, request.GET.get(TO_FIELD_VAR))
        if to_field and not self.to_field_allowed(request, to_field):
            raise DisallowedModelAdminToField(
                "The field %s cannot be referenced." % to_field
            )

        if request.method == "POST" and "_saveasnew" in request.POST:
            print("Saving as new")
            object_id = None

        add = object_id is None

        if add:
            if not self.has_add_permission(request):
                raise PermissionDenied
            obj = None

        else:
            obj = self.get_object(request, unquote(object_id), to_field)

            if request.method == "POST":
                if not self.has_change_permission(request, obj):
                    raise PermissionDenied
            else:
                if not self.has_view_or_change_permission(request, obj):
                    raise PermissionDenied

            if obj is None:
                return self._get_obj_does_not_exist_redirect(
                    request, self.opts, object_id
                )

        fieldsets = self.get_fieldsets(request, obj)
        ModelForm = self.get_form(
            request, obj, change=not add, fields=flatten_fieldsets(fieldsets)
        )
        if request.method == "POST":
            # raise Exception("Hello Form Submission!")
            form = ModelForm(request.POST, request.FILES, instance=obj)
            formsets, inline_instances = self._create_formsets(
                request,
                form.instance,
                change=not add,
            )
            form_validated = form.is_valid()
            if form_validated:
                new_object = self.save_form(request, form, change=not add)
            else:
                new_object = form.instance
            if all_valid(formsets) and form_validated:
                self.save_model(request, new_object, form, not add)
                self.save_related(request, form, formsets, not add)
                change_message = self.construct_change_message(
                    request, form, formsets, add
                )
                if add:
                    self.log_addition(request, new_object, change_message)
                    return self.response_add(request, new_object)
                else:
                    self.log_change(request, new_object, change_message)
                    return self.response_change(request, new_object)
            else:
                form_validated = False
        else:
            print("Hello Form Submission!")

            if add:
                initial = self.get_changeform_initial_data(request)
                form = ModelForm(initial=initial)
                formsets, inline_instances = self._create_formsets(
                    request, form.instance, change=False
                )
            else:
                form = ModelForm(instance=obj)
                formsets, inline_instances = self._create_formsets(
                    request, obj, change=True
                )

        if not add and not self.has_change_permission(request, obj):
            readonly_fields = flatten_fieldsets(fieldsets)
        else:
            readonly_fields = self.get_readonly_fields(request, obj)
        admin_form = helpers.AdminForm(
            form,
            list(fieldsets),
            # Clear prepopulated fields on a view-only form to avoid a crash.
            (
                self.get_prepopulated_fields(request, obj)
                if add or self.has_change_permission(request, obj)
                else {}
            ),
            readonly_fields,
            model_admin=self,
        )
        media = self.media + admin_form.media

        inline_formsets = self.get_inline_formsets(
            request, formsets, inline_instances, obj
        )
        for inline_formset in inline_formsets:
            media += inline_formset.media

        if add:
            title = _("Add %s")
        elif self.has_change_permission(request, obj):
            title = _("Change %s")
        else:
            title = _("View %s")
        context = {
            **self.admin_site.each_context(request),
            "title": title % self.opts.verbose_name,
            "subtitle": str(obj) if obj else None,
            "adminform": admin_form,
            "object_id": object_id,
            "original": obj,
            "is_popup": IS_POPUP_VAR in request.POST or IS_POPUP_VAR in request.GET,
            "to_field": to_field,
            "media": media,
            "inline_admin_formsets": inline_formsets,
            "errors": helpers.AdminErrorList(form, formsets),
            "preserved_filters": self.get_preserved_filters(request),
        }

        # Hide the "Save" and "Save and continue" buttons if "Save as New" was
        # previously chosen to prevent the interface from getting confusing.
        if (
            request.method == "POST"
            and not form_validated
            and "_saveasnew" in request.POST
        ):
            context["show_save"] = False
            context["show_save_and_continue"] = False
            # Use the change template instead of the add template.
            add = False

        context.update(extra_context or {})

        return self.render_change_form(
            request, context, add=add, change=not add, obj=obj, form_url=form_url
        )

        



palette_admin = PaletteAdminSite(name='palette_admin')
palette_admin.register(User)
palette_admin.register(Group)



from setuptools import setup, find_packages

setup(
    name="dj-palette",
    version="1.0.1.0",
    description="A Django admin system with tools for building custom admin interfaces and reusable components",
    long_description=open("README.md", encoding="utf-8").read(),
    long_description_content_type="text/markdown",
    author="Joel O. Tanko",
    author_email="7thogofe@gmail.com",
    url="https://github.com/ogofe/dj_palette",
    project_urls={
        "Documentation": "https://django-palette.readthedocs.io",
        "Issues": "https://github.com/ogofe/dj_palette/issues",
        "Repository": "https://github.com/ogofe/dj_palette",
    },
    packages=find_packages(include=['dj_palette', 'dj_palette.*']),
    include_package_data=True,
    package_data={
        'dj_palette': [
            'templates/**/*.html',
            'static/**/*',
            'migrations/**/*.py',
            'templatetags/**/*.py',
            'palette_admin/**/*',
            'palette_admin/templates/**/*.html',
            'palette_admin/static/**/*',
            'palette_admin/migrations/**/*.py',
        ],
    },
    install_requires=[
        "Django>=2.2",
    ],
    classifiers=[
        "Framework :: Django",
        "Framework :: Django :: 2.2",
        "Framework :: Django :: 3.0",
        "Framework :: Django :: 3.1",
        "Framework :: Django :: 3.2",
        "Framework :: Django :: 4.0",
        "Framework :: Django :: 4.1",
        "Framework :: Django :: 4.2",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "Programming Language :: Python :: 3.13",
        "Operating System :: OS Independent",
        "License :: OSI Approved :: MIT License",
    ],
    python_requires=">=3.7",
)



from django.contrib.admin import AdminSite
from django.urls import path, re_path
from .views import dashboard_view, dynamic_admin_page, edit_admin_page
from django.template.response import TemplateResponse
from django.contrib.auth import logout, login
from django.shortcuts import redirect
from datetime import datetime
from django.apps import apps
from django.http import Http404
from django.utils.module_loading import import_string
from django.contrib.auth import REDIRECT_FIELD_NAME
from django.contrib.auth.decorators import login_not_required
from django.utils.translation import gettext as _
from django.utils.translation import gettext_lazy
from django.views.decorators.cache import never_cache
from django.urls import NoReverseMatch, Resolver404, resolve, reverse, reverse_lazy
from django.http import Http404, HttpResponsePermanentRedirect, HttpResponseRedirect
from django.utils.decorators import method_decorator
from django.conf import settings
from .conf import (
    SETTINGS,
)



PALETTE_SETTINGS = getattr(settings, 'PALETTE_ADMIN_SETTINGS', SETTINGS)




class PaletteAdminSite(AdminSite):
    # make the following variables dynamic from settings, e.g

    site_header = PALETTE_SETTINGS.get('site_header', "Palette Admin")
    site_title = PALETTE_SETTINGS.get('site_title', "Palette Admin")
    index_title = PALETTE_SETTINGS.get('index_title', "Palette Admin")

    def get_urls(self):
        custom_urls = [
            path('login/', self.login, name='login'),
            path('logout/', self.logout, name='logout'),
            path('password_change/', self.password_change, name='password_change'),
            path('password_change/done/', self.password_change_done, name='password_change_done'),
            path('support_chat/', self.admin_view(self.support_chat_view), name='support_chat'),
            path('profile/', self.admin_view(self.profile_view), name='profile'),
            re_path(r'^(?P<app_label>\w+)/$', self.admin_view(self.app_index), name='app_list'),
        ] + super().get_urls()

        if PALETTE_SETTINGS.get('show_editor', True):
            custom_urls.extend([
                path('pages/', dashboard_view, name='custom-dashboard'),
                path('pages/edit/<slug:slug>/', edit_admin_page, name='edit-admin-page'),
                path('pages/<slug:slug>/', dynamic_admin_page, name='dynamic-admin-page'),
            ])
        if PALETTE_SETTINGS.get('custom_urls'):
            other_urls = PALETTE_SETTINGS.get('custom_urls', [])
            custom_urls.extend([
                path(url_path, self.admin_view(view_func), name) for (url_path, view_func, name)  in other_urls
            ])
        return custom_urls


    def logout(self, request, **kwargs):
        logout(request)
        return redirect('palette_admin:login') # Redirect to the login page after logout


    @method_decorator(never_cache)
    @login_not_required
    def login(self, request, extra_context=None):
        """
        Display the login form for the given HttpRequest.
        """
        if request.method == "GET" and self.has_permission(request):
            # Already logged-in, redirect to admin index
            index_path = reverse("palette_admin:index", current_app=self.name)
            return HttpResponseRedirect(index_path)

        # Since this module gets imported in the application's root package,
        # it cannot import models from other applications at the module level,
        # and django.contrib.admin.forms eventually imports User.
        from django.contrib.admin.forms import AdminAuthenticationForm
        from django.contrib.auth.views import LoginView

        context = {
            **self.each_context(request),
            "title": _("Log in"),
            "subtitle": None,
            "app_path": request.get_full_path(),
            "username": request.user.get_username(),
        }
        if (
            REDIRECT_FIELD_NAME not in request.GET
            and REDIRECT_FIELD_NAME not in request.POST
        ):
            context[REDIRECT_FIELD_NAME] = reverse("palette_admin:index", current_app=self.name)
        context.update(extra_context or {})

        defaults = {
            "extra_context": context,
            'next_page': 'palette_admin:index',
            "authentication_form": self.login_form or AdminAuthenticationForm,
            "template_name": self.login_template or "admin/login.html",
        }
        request.current_app = self.name
        return LoginView.as_view(**defaults)(request)


    def get_app_list(self, request):
        app_dict = self._build_app_dict(request)

        # Inject model manager as model.objects for template use
        for app_label, app in app_dict.items():
            for model_dict in app['models']:
                model_class = self._registry[model_dict['model']].model
                model_dict['objects'] = model_class._default_manager  # or .objects

        app_list = sorted(app_dict.values(), key=lambda x: x['name'].lower())
        for app in app_list:
            app['models'].sort(key=lambda x: x['name'])
        return app_list


    def each_context(self, request):
        context = super().each_context(request)
        context['custom_links'] = PALETTE_SETTINGS.get('custom_links', [])

        for key, value in PALETTE_SETTINGS.items():
            if key not in context:
                context[key] = value

        if PALETTE_SETTINGS.get('available_apps'):
            context['available_apps'] =  [app for app in self.get_app_list(request) if app['app_label'] in PALETTE_SETTINGS.get('available_apps') ]
        else:
            context['available_apps'] = self.get_app_list(request)
        return context
    

    # Dashboard index view
    def index(self, request, extra_context:dict=None, **kwargs):
        context:dict = self.each_context(request)
        
        now = datetime.now().time()
        greeting = 'Morning'
        if 12 < now.hour <= 16:greeting='Afternoon'
        if now.hour > 16:greeting='Evening'

        context.update({
            'greeting': greeting,
            'user': request.user,
            'app_list': self.get_app_list(request),
            # **extra_context if extra_context else 
        })
        
        return TemplateResponse(request, "admin/index.html", context)


    def app_index(self, request, app_label, extra_context=None):
        app_config = apps.get_app_config(app_label)
        if not app_config:
            raise Http404("App not found")

        context = {
            **self.each_context(request),
            "title": app_config.verbose_name,
            "app_label": app_label,
            "app_list": [app for app in self.get_app_list(request) if app['app_label'] == app_label],
        }
        context.update(extra_context or {})
        return TemplateResponse(request, "admin/app_index.html", context)


    def profile_view(self, request):
        """
        Custom view to handle profile viewing.
        """
        context = self.each_context(request)
        context['title'] = "User Profile"
        context['user'] = request.user
        return TemplateResponse(request, "admin/profile.html", context)


    def support_chat_view(self, request):
        """
        Custom view to handle profile viewing.
        """
        context = self.each_context(request)
        context['title'] = "User Profile"
        context['user'] = request.user
        return TemplateResponse(request, "admin/profile.html", context)
    



# Some other config



# dj_palette/templatetags/palette.py
"""
Palette Template Engine - Custom Django Template Tags
======================================================

A component-based template system for Django with support for:
- Reusable components (palette_component)
- Overridable slots (palette_block)
- Component composition (palette_ui)
- Context-aware rendering with proper scoping
"""

import os
import re
from django import template
from django.template import (
    Node,
    NodeList,
    TemplateSyntaxError,
    loader,
    Context as TempContext,
)
from django.template.base import Parser, Token, token_kwargs
from django.utils.safestring import mark_safe
from django.urls import reverse
from django.template.loader import render_to_string


register = template.Library()

@register.filter('dir')
def dir_filter(obj):
    return f"{dir(obj)}"


# ========================
# AST WALKING HELPERS
# ========================

def _iter_nodelist_recursive(nodelist):
    """
    Walk a NodeList recursively, yielding all nodes including children.
    
    Supports nested structures:
    - nodelist: standard child list
    - nodelist_true/nodelist_false: if/else branches
    - nodelist_loop: for loops
    """
    for node in nodelist:
        yield node
        for attr in ("nodelist", "nodelist_true", "nodelist_false", "nodelist_loop"):
            child = getattr(node, attr, None)
            if isinstance(child, NodeList):
                yield from _iter_nodelist_recursive(child)


# ========================
# PALETTE_OVERRIDE NODE
# ========================

class PaletteOverrideNode(Node):
    """
    Represents an override block for a palette_block slot.
    
    Structure:
        {% palette_override block_name %}
            ... override content ...
        {% endpalette_override %}
    
    The override content is not rendered directly; instead, it's collected
    by palette_ui and applied when the component is rendered.
    """

    def __init__(self, name, nodelist):
        self.name = name
        self.nodelist = nodelist

    def render(self, context):
        # Overrides are never rendered directly; they're processed by palette_ui
        return ""


@register.tag("palette_override")
def do_palette_override(parser, token):
    """Parse a palette_override tag."""
    bits = token.split_contents()
    if len(bits) != 2:
        raise TemplateSyntaxError(
            "palette_override requires exactly one argument: the block name"
        )
    
    block_name = bits[1]
    nodelist = parser.parse(("endpalette_override",))
    parser.delete_first_token()
    
    return PaletteOverrideNode(block_name, nodelist)


# ========================
# PALETTE_BLOCK NODE
# ========================

class PaletteBlockNode(Node):
    """
    Represents a named slot (block) within a component.
    
    Structure:
        {% palette_block card_header %}
            ... default content ...
        {% endpalette_block %}
    
    Rendering behavior:
    1. Render the base content (self.nodelist)
    2. Check if an override exists in context.render_context["palette_overrides"]
    3. If override exists, create an isolated context with block.super
    4. Render override with access to all parent variables
    
    This matches Django's {% block %} / {{ block.super }} semantics.
    """

    def __init__(self, block_name, nodelist):
        self.block_name = block_name
        self.nodelist = nodelist

    def render(self, context):
        """Render the palette_block with override support."""
        # Render the base content
        original = self.nodelist.render(context)
        
        # Check for overrides (set by palette_ui before rendering component)
        overrides = context.render_context.get("palette_overrides", {}) or {}
        
        if self.block_name not in overrides:
            # No override; return base content
            return original
        
        # Override exists; render it with block.super available
        override_nodelist = overrides[self.block_name]
        
        class _Block:
            """Provides block.super access to the original rendered content."""
            def __init__(self, sup):
                self.super = sup
        
        # Build isolated context preserving parent variables
        base = context.flatten()
        temp_ctx = TempContext(base)
        temp_ctx["block"] = _Block(original)
        
        return override_nodelist.render(temp_ctx)


@register.tag("palette_block")
def palette_block(parser, token):
    """
    Parse a palette_block tag.
    
    Syntax:
        {% palette_block name %}...{% endpalette_block %}
    
    Note: NO output during parsing. Parsing only constructs the AST.
    """
    bits = token.split_contents()
    if len(bits) != 2:
        raise TemplateSyntaxError(
            "palette_block takes exactly one argument: the block name"
        )
    
    block_name = bits[1]
    nodelist = parser.parse(("endpalette_block",))
    parser.delete_first_token()
    
    return PaletteBlockNode(block_name, nodelist)


# ========================
# PALETTE_COMPONENT NODE
# ========================

class PaletteComponentNode(Node):
    """
    Represents a component definition.
    
    Structure:
        {% palette_component "card" %}
            {% palette_block card_header %}...{% endpalette_block %}
            {% palette_block card_body %}...{% endpalette_block %}
        {% endpalette_component %}
    
    The component's nodelist is registered into context.render_context
    during the first render pass for later lookup by palette_ui.
    
    When rendered directly (not via palette_ui), returns the default
    rendered output so component definition files can be previewed.
    """

    def __init__(self, file_key, component_name, nodelist):
        """
        Args:
            file_key: Template filename (e.g., 'palette/components/card.html')
            component_name: Component identifier (e.g., 'card')
            nodelist: AST node list (may contain palette_block nodes)
        """
        self.file_key = file_key
        self.component_name = component_name
        self.nodelist = nodelist

    def render(self, context):
        """
        1. Register component nodelist into PALETTE_COMPONENTS registry
        2. Render the default content (for preview)
        """
        # Register into render_context for later lookup
        registry = context.render_context.setdefault("PALETTE_COMPONENTS", {})
        file_components = registry.setdefault(self.file_key, {})
        file_components[self.component_name] = self.nodelist
        
        # print(f"[PALETTE_COMPONENT] Registered '{self.component_name}' in '{self.file_key}'")
        
        # Expose registry to templates for debugging
        context["PALETTE_COMPONENTS"] = registry
        
        # Render default content
        return self.nodelist.render(context)


@register.tag("palette_component")
def palette_component(parser, token):
    """
    Parse a palette_component tag.
    
    Syntax:
        {% palette_component "component_name" %}...{% endpalette_component %}
    
    The component_name is extracted from quotes or used as-is.
    File key is determined from parser.origin (template filename).
    """
    bits = token.split_contents()
    if len(bits) != 2:
        raise TemplateSyntaxError(
            "palette_component requires exactly one argument: the component name"
        )
    
    raw = bits[1]
    
    # Handle quoted strings
    if raw and raw[0] in ('"', "'") and raw[-1] == raw[0]:
        component_name = raw[1:-1]
    else:
        component_name = raw
    
    # Determine template file key
    file_key = (
        getattr(parser.origin, "template_name", None) or
        getattr(parser.origin, "name", None) or
        "<unknown>"
    )
    
    # Parse the component's nodelist (includes nested palette_block nodes)
    nodelist = parser.parse(("endpalette_component",))
    parser.delete_first_token()
    
    return PaletteComponentNode(file_key, component_name, nodelist)


# ========================
# PALETTE_UI NODE
# ========================

def _parse_quoted_value(val_str):
    """
    Parse a prop value which can be:
    1. "literal string" -> ("literal", "literal string")
    2. "{{ variable }}" -> ("template_expr", "variable")
    3. "{% template_tag %}" -> ("tag", "template_tag")
    4. bare_variable -> ("expression", "bare_variable")
    5. Mixed: "prefix {{ var }} suffix" -> ("template_expr", "prefix {{ var }} suffix")
    
    Returns tuple of (type, content):
    - type: "literal", "template_expr", "tag", or "expression"
    - content: the extracted/normalized content
    
    Crucially:
    - Only treat as template_expr if wrapped in {{ }}
    - Only treat as tag if wrapped in {% %}
    - Otherwise treat as literal (even if unquoted)
    """
    val_str = val_str.strip()
    
    if not val_str:
        return ("literal", "")
    
    # Case 1: Quoted string - extract the inner content
    if val_str[0] in ('"', "'"):
        quote_char = val_str[0]
        # Find the matching closing quote
        end_idx = val_str.rfind(quote_char)
        
        if end_idx > 0:  # Found closing quote
            inner = val_str[1:end_idx]
            
            # Check what's inside the quotes
            # 1a: "{{ variable }}" -> template expression
            if inner.startswith("{{") and inner.endswith("}}"):
                expr_content = inner[2:-2].strip()
                return ("template_expr", expr_content)
            
            # 1b: "{% template_tag %}" -> template tag
            elif inner.startswith("{%") and inner.endswith("%}"):
                return ("tag", inner)
            
            # 1c: Mixed content with {{ }}: "prefix {{ var }} suffix"
            elif "{{" in inner and "}}" in inner:
                return ("template_expr", inner)
            
            # 1d: Regular quoted literal (including strings with spaces)
            else:
                return ("literal", inner)
        else:
            # Mismatched quotes
            return ("literal", val_str)
    
    # Case 2: Unquoted value
    # Check for template expression: {{ ... }}
    if val_str.startswith("{{") and val_str.endswith("}}"):
        expr_content = val_str[2:-2].strip()
        return ("template_expr", expr_content)
    
    # Check for template tag: {% ... %}
    if val_str.startswith("{%") and val_str.endswith("%}"):
        return ("tag", val_str)
    
    # Check for mixed expression: "prefix {{ var }} suffix" without quotes
    if "{{" in val_str and "}}" in val_str:
        return ("template_expr", val_str)
    
    # Case 3: Bare variable or literal
    # Treat as "expression" - could be a variable reference (e.g., stats.total_users)
    # The parser will later decide if it's a valid variable reference or just a string
    return ("expression", val_str)


class PaletteUINode(Node):
    """
    Renders a component instance with context variables and overrides.
    
    Structure:
        {% palette_ui file="path/to/components.html" component="card" with title=page.title %}
            {% palette_override card_header %}
                <h2>{{ title }}</h2>
            {% endpalette_override %}
        {% endpalette_ui %}
    
    Execution flow:
    1. Resolve file and component names from expressions
    2. Resolve context variables (via 'with')
    3. Locate component nodelist in PALETTE_COMPONENTS registry
    4. Collect overrides from inner palette_override nodes
    5. Push a new context layer with props
    6. Set overrides in render_context
    7. Render component nodelist
    8. Restore context (pop) and clean up overrides
    """

    def __init__(self, reserved, props, nodelist):
        """
        Args:
            reserved: dict of file/component FilterExpression or string literals
            props: dict of variable name -> FilterExpression
            nodelist: inner nodes (contains palette_override tags)
        """
        self.reserved = reserved
        self.props = props
        self.nodelist = nodelist

    def _resolve_token(self, expr, context):
        """
        Safely resolve a FilterExpression to a value.
        
        Returns None if resolution fails.
        """
        if expr is None:
            return None
        try:
            return expr.resolve(context)
        except Exception:
            return None

    def _find_component_in_registry(self, registry, tpl_name, comp_name):
        """
        Attempt to locate a component in the registry using multiple fallback strategies.
        
        Returns the component nodelist if found, None otherwise.
        """
        # Strategy 1: Direct lookup
        comp_nodelist = registry.get(tpl_name, {}).get(comp_name)
        if comp_nodelist is not None:
            return comp_nodelist
        
        # Strategy 2: Template object name match
        try:
            template_obj = loader.get_template(tpl_name)
            tpl_obj_name = (
                getattr(template_obj, "name", None) or
                getattr(getattr(template_obj, "origin", None), "name", None)
            )
            if tpl_obj_name:
                comp_nodelist = registry.get(tpl_obj_name, {}).get(comp_name)
                if comp_nodelist is not None:
                    return comp_nodelist
        except Exception:
            pass
        
        # Strategy 3: Basename match
        basename = os.path.basename(tpl_name)
        for key in list(registry.keys()):
            if os.path.basename(key) == basename:
                comp_nodelist = registry.get(key, {}).get(comp_name)
                if comp_nodelist is not None:
                    return comp_nodelist
        
        # Strategy 4: Scan template AST for component definition
        try:
            template_obj = loader.get_template(tpl_name)
            root_nodelist = (
                getattr(template_obj, "nodelist", None) or
                (hasattr(template_obj, "template") and
                 getattr(template_obj.template, "nodelist", None))
            )
            if root_nodelist:
                for node in _iter_nodelist_recursive(root_nodelist):
                    if (isinstance(node, PaletteComponentNode) and
                        node.component_name == comp_name):
                        # Register for future lookups
                        registry.setdefault(tpl_name, {})[comp_name] = node.nodelist
                        return node.nodelist
        except Exception:
            pass
        
        return None

    def render(self, context):
        """
        Render the palette_ui component with context variables and overrides.
        
        Process:
        1. Resolve file path and component name from expressions/literals
        2. Load template and trigger component registration if needed
        3. Resolve all context variables passed via 'with' clause
        4. Collect overrides from inner palette_override nodes
        5. Push context layer with component props
        6. Set overrides in render_context
        7. Render component nodelist
        8. Pop context and restore previous state
        """
        # Resolve file and component names
        file_expr = self.reserved.get("file")
        comp_expr = self.reserved.get("component")
        
        if file_expr is None or comp_expr is None:
            print("[PALETTE_UI] ERROR: Missing file or component expression")
            return ""
        
        # Resolve file path: can be string literal or FilterExpression ({{ var }})
        tpl_name = self._resolve_token(file_expr, context)
        if isinstance(file_expr, str):  # literal string
            tpl_name = file_expr
        
        # Resolve component name: can be string literal or FilterExpression ({{ var }})
        if isinstance(comp_expr, str):  # literal string
            comp_name = comp_expr
        else:
            comp_name = self._resolve_token(comp_expr, context)
        
        if not tpl_name or not comp_name:
            print(f"[PALETTE_UI] ERROR: Invalid file='{tpl_name}' or component='{comp_name}'")
            return f"<!-- palette_ui: file='{tpl_name}' component='{comp_name}' -->"
        
        # Get or create component registry
        registry = context.render_context.setdefault("PALETTE_COMPONENTS", {})
        
        # Ensure template is loaded, which will trigger component registration
        try:
            template_obj = loader.get_template(tpl_name)
        except Exception as e:
            print(f"[PALETTE_UI] ERROR: Could not load template '{tpl_name}': {e}")
            return f"<!-- palette_ui: template '{tpl_name}' not found -->"
        
        # Locate component nodelist in registry
        comp_nodelist = self._find_component_in_registry(registry, tpl_name, comp_name)
        
        if comp_nodelist is None:
            print(f"[PALETTE_UI] ERROR: Component '{comp_name}' not found in '{tpl_name}'")
            return f"<!-- palette_ui: component '{comp_name}' not found -->"
        
        # Resolve context variables from 'with' clause
        # Each value is either:
        # - ("literal", string_value) -> use as-is
        # - ("expression", FilterExpression) -> resolve from context
        # - ("tag", template_tag_str) -> would need special handling
        local_vars = {}
        for var_name, var_spec in self.props.items():
            try:
                if isinstance(var_spec, tuple) and len(var_spec) == 2:
                    var_type, var_value = var_spec
                    if var_type == "literal":
                        # Literal string value
                        local_vars[var_name] = var_value
                    elif var_type == "expression":
                        # FilterExpression - resolve from context
                        resolved_value = var_value.resolve(context)
                        local_vars[var_name] = resolved_value
                    elif var_type == "tag":
                        # Template tag - for now, pass as-is
                        local_vars[var_name] = var_value
                    else:
                        local_vars[var_name] = None
                else:
                    # Fallback: assume it's a FilterExpression for backward compatibility
                    resolved_value = var_spec.resolve(context)
                    local_vars[var_name] = resolved_value
            except Exception as e:
                print(f"[PALETTE_UI] WARNING: Could not resolve {var_name}: {e}")
                local_vars[var_name] = None
        
        # Collect overrides from inner palette_override nodes
        overrides = {}
        for node in _iter_nodelist_recursive(self.nodelist):
            if isinstance(node, PaletteOverrideNode):
                # Strip quotes from override block name if needed
                block_name = node.name
                if block_name and block_name[0] in ('"', "'") and block_name[-1] == block_name[0]:
                    block_name = block_name[1:-1]
                overrides[block_name] = node.nodelist
        
        print(f"[PALETTE_UI] Rendering component='{comp_name}' from file='{tpl_name}'")
        
        # Save previous state (for nested palette_ui calls)
        prev_overrides = context.render_context.get("palette_overrides", None)
        prev_components = context.render_context.get("palette_current_component", None)
        
        # Set up render_context for palette_block nodes to access
        context.render_context["palette_overrides"] = overrides
        context.render_context["palette_current_component"] = comp_name
        
        # Push a new context layer for component props (matches Django's {% include %} tag)
        context.push()
        try:
            # Inject component props into the new context layer
            for var_name, var_value in local_vars.items():
                context[var_name] = var_value
            
            # Expose registry for debugging
            context["PALETTE_COMPONENTS"] = registry
            context["component_name"] = comp_name
            
            # Render the component nodelist with new context and overrides
            rendered = comp_nodelist.render(context)
            
        except Exception as e:
            import traceback
            traceback.print_exc()
            rendered = f"<!-- palette_ui render error: {e} -->"
        finally:
            # Pop the context layer
            context.pop()
            
            # Restore previous state
            context.render_context["palette_overrides"] = prev_overrides
            context.render_context["palette_current_component"] = prev_components
                    
        return mark_safe(rendered)


@register.tag("palette_ui")
def do_palette_ui(parser, token):
    """
    Parse a palette_ui tag.
    
    Grammar:
      {% palette_ui file="path/to/components.html" component="card_name" with title=page.title count=total %}
         {% palette_override card_header %}
            <h2>{{ title }}</h2>
         {% endpalette_override %}
      {% endpalette_ui %}
    
    Supports:
    - file="..." or file=variable_name for template path
    - component="..." or component=variable_name for component name
    - with key1=value1 key2=value2 ... for context variables
      - value can be: "literal_string", "{{ variable }}", "{% url ... %}", variable_name
    - Inner palette_override blocks for slot overrides
    """
    bits = token.split_contents()
    tag_name = bits[0]
    
    if len(bits) < 3:
        raise TemplateSyntaxError(
            f"{tag_name} requires at least 'file=' and 'component=' keyword arguments. "
            f"Syntax: {{% {tag_name} file=\"path\" component=\"name\" [with key=value ...] %}}"
        )
    
    # Parse keyword arguments
    file_expr = None
    comp_expr = None
    with_vars = {}
    
    i = 1
    while i < len(bits):
        bit = bits[i]
        
        # Stop at 'with' keyword
        if bit == "with":
            i += 1
            break
        
        # Must be a keyword argument (key=value)
        if "=" not in bit:
            raise TemplateSyntaxError(
                f"{tag_name}: Expected keyword argument (key=value), got '{bit}'"
            )
        
        key, val = bit.split("=", 1)
        
        # Parse the value
        val_type, val_content = _parse_quoted_value(val)
        
        # Handle reserved keywords
        if key == "file":
            if val_type == "literal":
                # Literal path
                file_expr = val_content
            elif val_type == "template_expr":
                # Template expression: compile as filter
                file_expr = parser.compile_filter(val_content)
            elif val_type == "tag":
                # Template tag - compile
                file_expr = parser.compile_filter(val_content)
            else:  # "expression"
                # Bare variable
                file_expr = parser.compile_filter(val_content)
        
        elif key == "component":
            if val_type == "literal":
                comp_expr = val_content
            elif val_type == "template_expr":
                comp_expr = parser.compile_filter(val_content)
            elif val_type == "tag":
                comp_expr = parser.compile_filter(val_content)
            else:  # "expression"
                comp_expr = parser.compile_filter(val_content)
        
        else:
            # Treat as a 'with' variable before 'with' keyword is seen
            if val_type == "literal":
                # Literal value - store as-is, will be used as string
                with_vars[key] = ("literal", val_content)
            elif val_type == "template_expr":
                # Template expression - compile as filter
                with_vars[key] = ("expression", parser.compile_filter(val_content))
            elif val_type == "tag":
                # Template tag - we'll handle this specially
                with_vars[key] = ("tag", val_content)
            else:  # "expression"
                # Bare variable reference - compile as filter
                with_vars[key] = ("expression", parser.compile_filter(val_content))
        
        i += 1
    
    # Parse remaining 'with' variables
    while i < len(bits):
        bit = bits[i]
        if "=" not in bit:
            raise TemplateSyntaxError(
                f"{tag_name}: Expected 'key=value' in with clause, got '{bit}'"
            )
        
        key, val = bit.split("=", 1)
        val_type, val_content = _parse_quoted_value(val)
        
        if val_type == "literal":
            with_vars[key] = ("literal", val_content)
        elif val_type == "template_expr":
            with_vars[key] = ("expression", parser.compile_filter(val_content))
        elif val_type == "tag":
            with_vars[key] = ("tag", val_content)
        else:  # "expression"
            with_vars[key] = ("expression", parser.compile_filter(val_content))
        i += 1
    
    # Validate required arguments
    if file_expr is None:
        raise TemplateSyntaxError(f"{tag_name} requires 'file=' argument")
    if comp_expr is None:
        raise TemplateSyntaxError(f"{tag_name} requires 'component=' argument")
    
    # Parse inner content (palette_override blocks)
    inner_nodelist = parser.parse(("endpalette_ui",))
    parser.delete_first_token()
    
    # Store reserved keywords (file, component) separately from props
    reserved = {
        "file": file_expr,
        "component": comp_expr,
    }
    
    # print(f"[PALETTE_UI PARSER] Parsed tag with file={file_expr!r}, component={comp_expr!r}")
    # print(f"[PALETTE_UI PARSER] With vars: {list(with_vars.keys())}")
    
    return PaletteUINode(reserved, with_vars, inner_nodelist)



# ========================
# UTILITY TAGS & FILTERS
# ========================

@register.simple_tag(takes_context=True, name="back_button")
def back_button(context):
    path = context['request'].path
    # Matches /admin/app/model/add/
    m = re.match(r'^/admin/([^/]+)/([^/]+)/add/$', path)
    if m:
        app, model = m.groups()
        return f"/admin/{app}/{model}/"
    # Matches /admin/app/model/pk/change/
    m = re.match(r'^/admin/([^/]+)/([^/]+)/\d+/change/$', path)
    if m:
        app, model = m.groups()
        return f"/admin/{app}/{model}/"
    # Matches /admin/app/model/
    m = re.match(r'^/admin/([^/]+)/([^/]+)/$', path)
    if m:
        app = m.group(1)
        return f"/admin/{app}/"
    # Default: admin index
    return reverse('palette_admin:index')


@register.filter(name='render_ui')
def render_ui(component_name, props_str=""):
    props_dict = {}
    try:
        for item in props_str.split(";"):
            if item.strip():
                key, val = item.split(":", 1)
                props_dict[key.strip()] = val.strip()
    except Exception as e:
        props_dict["error"] = str(e)
    return render_to_string(f"palette/components/{component_name}.html", {"props": props_dict})

@register.filter
def widget_type(field):
    print("Widget for field:", field.field.widget_type)
    return field.field.widget_type


@register.filter
def is_multiple_select(field):
    from django.forms.models import ModelMultipleChoiceField
    if isinstance(field.field.field, ModelMultipleChoiceField):
        return True
    return False


@register.filter(name='admin_fields')
def admin_fields(obj):
    """
    Returns a list of (field_name, field_value) tuples for an object.
    Useful for displaying object details in admin interfaces.
    
    Example:
        {% for field_name, field_value in obj|admin_fields %}
            <span>{{ field_name }}: {{ field_value }}</span>
        {% endfor %}
    """
    from django.contrib.admin.utils import label_for_field
    from django.db import models
    
    if not obj:
        return []
    
    result = []
    model = obj.__class__
    
    # Get all fields from the model
    for field in model._meta.get_fields():
        # Skip many-to-many and reverse relations
        if field.many_to_one or field.one_to_one:
            try:
                value = getattr(obj, field.name, None)
                result.append((field.verbose_name.title(), value))
            except Exception:
                pass
        elif not (field.many_to_many or field.one_to_many):
            try:
                value = getattr(obj, field.name, None)
                result.append((field.verbose_name.title(), value))
            except Exception:
                pass
    
    return result


@register.filter(name='admin_field')
def admin_field(obj, field_name):
    """
    Get a specific field value from an object.
    
    Example:
        {{ obj|admin_field:"email" }}
    """
    try:
        return getattr(obj, field_name, None)
    except Exception:
        return None










